@@ -16,6 +16,7 @@ class DatabaseManager:
     
     def __init__(self, sigmund, username: str,
                  encryption_key: [str, bytes]=None):
+        self.transient_settings = {}
         self._sigmund = sigmund
         self.username = username
         self.encryption_manager = EncryptionManager(encryption_key)
@@ -144,32 +145,138 @@ class DatabaseManager:
         conversation_data = json.loads(decrypted_data)
         conversation_data['message_history'] = self.get_message_history(
             conversation_data)
-        return conversation_data
+        return conversation_data            
 
-    def set_active_conversation(self, conversation_id: int) -> bool:
+    def _get_conversation(self, conversation_id: int):
+        """Retrieves a conversation for the current user.
+
+        Parameters
+        ----------
+        conversation_id : int
+            The ID of the conversation to retrieve
+
+        Returns
+        -------
+        Conversation or None
+            The Conversation object if found and belongs to the user, None otherwise
+        """
         try:
-            # We first get the active conversation
             conversation = Conversation.query.filter_by(
                 conversation_id=conversation_id, user_id=self.user_id).one()
+            return conversation
         except NoResultFound:
             logger.warning(f"Conversation {conversation_id} not found or does "
                            f"not belong to user {self.user_id}")
-            return False
-        # And then update the current time for that conversation so that
-        # it ends on top
-        decrypted_data = self.encryption_manager.decrypt_data(
-            conversation.data)
-        conversation_data = json.loads(decrypted_data)
-        conversation_data['last_updated'] = time.time()
+            return None
+
+    def _decrypt_conversation_data(self, conversation):
+        """Decrypts and parses conversation data.
+
+        Parameters
+        ----------
+        conversation : Conversation
+            The conversation object with encrypted data
+
+        Returns
+        -------
+        dict
+            The decrypted and parsed conversation data as a dictionary
+        """
+        decrypted_data = self.encryption_manager.decrypt_data(conversation.data)
+        return json.loads(decrypted_data)
+
+    def _encrypt_conversation_data(self, conversation_data: dict):
+        """Encrypts conversation data.
+
+        Parameters
+        ----------
+        conversation_data : dict
+            The conversation data dictionary to encrypt
+
+        Returns
+        -------
+        bytes
+            The encrypted data
+        """
         json_data = json.dumps(conversation_data)
-        encrypted_data = self.encryption_manager.encrypt_data(
-            json_data.encode('utf-8'))
+        return self.encryption_manager.encrypt_data(json_data.encode('utf-8'))
+
+    def _update_conversation_data(self, conversation, conversation_data: dict):
+        """Updates and saves encrypted conversation data.
+
+        Parameters
+        ----------
+        conversation : Conversation
+            The conversation object to update
+        conversation_data : dict
+            The updated conversation data dictionary
+        """
+        encrypted_data = self._encrypt_conversation_data(conversation_data)
         conversation.data = encrypted_data
-        # Finally we change the active conversation for the user
+        db.session.commit()
+
+    def set_active_conversation(self, conversation_id: int) -> bool:
+        """Sets the active conversation for the current user.
+
+        This method retrieves the specified conversation, updates its last_updated
+        timestamp, encrypts the updated data, and sets it as the user's active
+        conversation.
+
+        Parameters
+        ----------
+        conversation_id : int
+            The ID of the conversation to set as active
+
+        Returns
+        -------
+        bool
+            True if the conversation was successfully set as active, False if
+            the conversation was not found or doesn't belong to the user
+        """
+        conversation = self._get_conversation(conversation_id)
+        if conversation is None:
+            return False
+    
+        # Update the last_updated timestamp
+        conversation_data = self._decrypt_conversation_data(conversation)
+        conversation_data['last_updated'] = time.time()
+        self._update_conversation_data(conversation, conversation_data)
+    
+        # Change the active conversation for the user
         user = User.query.filter_by(user_id=self.user_id).one()
         user.active_conversation_id = conversation.conversation_id
         db.session.commit()
         return True
+
+    def set_conversation_title(self, conversation_id: int, title: str) -> bool:
+        """Updates the title of a conversation.
+
+        This method retrieves the specified conversation, decrypts its data,
+        updates the title field, and re-encrypts the data before committing
+        to the database.
+
+        Parameters
+        ----------
+        conversation_id : int
+            The ID of the conversation to update
+        title : str
+            The new title for the conversation
+
+        Returns
+        -------
+        bool
+            True if the title was successfully updated, False if the
+            conversation was not found or doesn't belong to the user
+        """
+        conversation = self._get_conversation(conversation_id)
+        if conversation is None:
+            return False
+    
+        # Update the title
+        conversation_data = self._decrypt_conversation_data(conversation)
+        conversation_data['title'] = title
+        self._update_conversation_data(conversation, conversation_data)
+        return True
     
     def update_active_conversation(self, conversation_data: dict) -> bool:
         """Updates the active conversation. The messages are saved in a 
@@ -185,7 +292,7 @@ class DatabaseManager:
         except NoResultFound:
             logger.warning(
                 f"No active conversation to update for user {self.user_id}")
-            return False            
+            return            
         logger.info(f'updating conversation {user.active_conversation_id}')
         logger.info('deleting old messages')
         # Batch delete all messages linked to this conversation, because we
@@ -211,7 +318,7 @@ class DatabaseManager:
         logger.info('committing conversation')
         db.session.commit()
         logger.info('done')
-        return True
+        return conversation.conversation_id
 
     def list_conversations(self, query=None) -> dict:
         conversations = {}
@@ -326,68 +433,74 @@ class DatabaseManager:
             .scalar()
         return total_tokens if total_tokens is not None else 0
 
-    def update_subscription(self, stripe_customer_id: str,
-                            stripe_subscription_id: str, from_date=None,
-                            to_date=None):
-        """Updates a subscription. By default the subscriptions starts 
-        immediately and ends exactly one month from the current time. A user
-        has only one subscription, which means that a subscription should be
-        added if it doesn't exist, and updated if it already exists.
+    def add_subscription_record(self, stripe_customer_id: str,
+                                stripe_subscription_id: str,
+                                from_date=None, to_date=None):
+        """Always append a new subscription record. Do not update old rows.
+        from_date defaults to now; to_date defaults to now + subscription_length.
         """
         now = datetime.utcnow()
         from_date = from_date or now
         to_date = to_date or now + timedelta(days=config.subscription_length)
-        subscription = Subscription.query.filter_by(
-            user_id=self.user_id).first()
-        if subscription:
-            subscription.from_date = from_date
-            subscription.to_date = to_date
-        else:
-            subscription = Subscription(
-                user_id=self.user_id, from_date=from_date, to_date=to_date,
-                stripe_customer_id=stripe_customer_id,
-                stripe_subscription_id=stripe_subscription_id)
-            db.session.add(subscription)
+        subscription = Subscription(
+            user_id=self.user_id,
+            from_date=from_date,
+            to_date=to_date,
+            stripe_customer_id=stripe_customer_id,
+            stripe_subscription_id=stripe_subscription_id
+        )
+        db.session.add(subscription)
         db.session.commit()
 
+    def update_subscription(self, stripe_customer_id: str,
+                            stripe_subscription_id: str, from_date=None,
+                            to_date=None):
+        """Backwards-compatible wrapper that appends a new subscription record."""
+        return self.add_subscription_record(
+            stripe_customer_id=stripe_customer_id,
+            stripe_subscription_id=stripe_subscription_id,
+            from_date=from_date,
+            to_date=to_date
+        )
+
     def check_subscription(self) -> bool:
-        """Returns a bool indicating whether the user is currently subscribed.
-        """
+        """Return True if at least one subscription row is currently active."""
         now = datetime.utcnow()
-        subscription = Subscription.query.filter_by(
-            user_id=self.user_id).first()
-        return subscription and \
-            subscription.from_date <= now < subscription.to_date
-
-    def cancel_subscription(self):
-        """Sets the to_date of the subscription to the current time if the user
-        is currently subscribed.
-        """
-        if not self.check_subscription():
-            return
-        subscription = Subscription.query.filter_by(
-            user_id=self.user_id).first()
-        subscription.to_date = datetime.utcnow()
-        db.session.commit()
+        # Efficient existence check
+        return db.session.query(
+            db.exists().where(
+                (Subscription.user_id == self.user_id) &
+                (Subscription.from_date <= now) &
+                (Subscription.to_date > now)
+            )
+        ).scalar()
 
     def get_stripe_customer_id(self) -> str:
-        subscription_record = Subscription.query.filter(
-            Subscription.user_id == self.user_id
-        ).order_by(Subscription.from_date.desc()).first()
-        if subscription_record:
-            return subscription_record.stripe_customer_id
-        return None
+        """Return the stripe_customer_id of the subscription with the latest to_date.
+        If multiple share the same to_date, pick the latest from_date as tie-breaker.
+        """
+        subscription_record = (
+            Subscription.query
+            .filter(Subscription.user_id == self.user_id)
+            .order_by(Subscription.to_date.desc(), Subscription.from_date.desc())
+            .first()
+        )
+        return subscription_record.stripe_customer_id if subscription_record else None
 
     @staticmethod
-    def from_stripe_customer_id(stripe_customer_id: str) -> str:
-        subscription_record = Subscription.query.filter(
-            Subscription.stripe_customer_id == stripe_customer_id
-        ).order_by(Subscription.from_date.desc()).first()
+    def from_stripe_customer_id(stripe_customer_id: str):
+        """Return a DatabaseManager instance for the user most recently associated
+        with this stripe_customer_id (based on latest to_date, then from_date).
+        """
+        subscription_record = (
+            Subscription.query
+            .filter(Subscription.stripe_customer_id == stripe_customer_id)
+            .order_by(Subscription.to_date.desc(), Subscription.from_date.desc())
+            .first()
+        )
         if not subscription_record:
             return None
-        user_record = User.query.filter(
-            User.user_id == subscription_record.user_id
-        ).first()
+        user_record = User.query.filter(User.user_id == subscription_record.user_id).first()
         if not user_record:
             return None
         return DatabaseManager(None, username=user_record.username)
@@ -398,15 +511,21 @@ class DatabaseManager:
         value as specified in the config or None if no default has been 
         specified.
         """
+        if key in self.transient_settings:
+            return self.transient_settings[key]
         setting = db.session.query(Setting).filter_by(
             user_id=self.user_id, key=key).first()
         return setting.value if setting \
             else config.settings_default.get(key, None)
 
-    def set_setting(self, key: str, value: str):
+    def set_setting(self, key: str, value: str, transient: bool = False):
         """Set a setting to specified value for the current user. If the
-        setting already exists, overwrite it.
+        setting already exists, overwrite it. When transient is True, the 
+        setting is not stored in the database, but only used for this session.
         """
+        if transient:
+            self.transient_settings[key] = value
+            return
         setting = Setting.query.filter_by(
             user_id=self.user_id, key=key).first()
         if setting:
@@ -424,4 +543,5 @@ class DatabaseManager:
             user_id=self.user_id).all()
         for setting in setting_objs:
             settings[setting.key] = setting.value
+        settings.update(self.transient_settings)
         return settings
