@@ -1,9 +1,8 @@
 import logging
 import uuid
 from cryptography.fernet import InvalidToken
+import multiprocessing as mp
 from . import prompt, config, utils
-from langchain.schema import HumanMessage, AIMessage, SystemMessage, \
-    FunctionMessage
 logger = logging.getLogger('sigmund')
 
 
@@ -12,6 +11,7 @@ class Messages:
     def __init__(self, sigmund, persistent=False):
         self._sigmund = sigmund
         self._persistent = persistent
+        self._conversation_id = None
         self.workspace_content = None
         self.workspace_language = None
         if self._persistent:
@@ -94,26 +94,19 @@ class Messages:
         """
         if system_prompt is None:
             system_prompt = self._system_prompt()
-        model_prompt = [SystemMessage(content=system_prompt)]
+        model_prompt = [dict(role='system', content=system_prompt)]
         for msg_nr, (role, content) in enumerate(
                 self._condensed_message_history):
             content = utils.remove_masked_elements(content)
-            if role == 'assistant':
-                model_prompt.append(AIMessage(content=content))
-            elif role == 'user':
+            if role == 'user':
                 # Prefix the last message with the current workspace
                 if self.workspace_content and \
                         msg_nr == len(self._condensed_message_history) - 1:
                     content = prompt.render(
-                        prompt.CURRENT_WORKPACE,
+                        prompt.CURRENT_WORKSPACE,
                         workspace_content=self.workspace_content,
                         workspace_language=self.workspace_language) + content
-                model_prompt.append(HumanMessage(content=content))
-            elif role == 'tool':
-                model_prompt.append(FunctionMessage(content=content,
-                                                    name='tool_function'))
-            else:
-                raise ValueError(f'Invalid role: {role}')
+            model_prompt.append(dict(role=role, content=content))
         return model_prompt
     
     def visible_messages(self):
@@ -133,15 +126,17 @@ class Messages:
         
     def _condense_message_history(self):
         system_prompt = self._system_prompt()
-        messages = [{"role": "system", "content": system_prompt}]
-        # Determine the effective length of the uncondensed prompt while
-        # excluding masked content
+        # Determine the effective length of the condensed prompt while
+        # excluding masked content. Makes content corresponds for example to
+        # image data, and other kinds of data that are shown to the user but not
+        # passed to the model.
         prompt_length = sum([
             len(utils.remove_masked_elements(content))
             for role, content in self._condensed_message_history
         ])
         logger.info(f'system prompt length: {len(system_prompt)}')
-        logger.info(f'prompt length (without system prompt): {prompt_length}')
+        logger.info(
+            f'condensed prompt length (without system prompt): {prompt_length}')
         if prompt_length <= config.max_prompt_length:
             logger.info('no need to condense')
             return
@@ -162,14 +157,20 @@ class Messages:
                 f'You said: {content}' if role == 'system' 
                 else f'User said: {content}'
                 for role, content in condense_messages))
-        self._condensed_text = self._sigmund.condense_model.predict(
-            condense_prompt)
+        result = self._sigmund.condense_model.predict(condense_prompt)
+        # Just in case a model gets confused and returns a tool call.
+        if not isinstance(result, str):
+            logger.error(f'condense model returned non-string result: {result}')
+            return
+        self._condensed_text = result.strip()
         
     def _system_prompt(self):
         """The system prompt that is used for question answering consists of
         several fragments.
         """
         system_prompt = [prompt.SYSTEM_PROMPT_IDENTITY]
+        if self._sigmund.transient_system_prompt:
+            system_prompt.append(self._sigmund.transient_system_prompt)
         # If available, documentation is also included in the prompt
         if len(self._sigmund.documentation):
             system_prompt.append(self._sigmund.documentation.prompt())
@@ -181,22 +182,7 @@ class Messages:
                 prompt.SYSTEM_PROMPT_CONDENSED,
                 summary=self._condensed_text))
         # Combine all non-empty prompt chunks
-        return '\n\n'.join(chunk for chunk in system_prompt if chunk.strip())
-        
-    def _update_title(self):
-        """The conversation title is updated when there are at least two 
-        messages, excluding the system prompt and AI welcome message. Based on
-        the last messages, a summary title is then created.
-        """
-        if len(self) <= 2 or \
-                self._conversation_title != config.default_conversation_title:
-            return
-        title_prompt = [SystemMessage(content=prompt.TITLE_PROMPT)]
-        title_prompt += self.prompt()[2:]
-        self._conversation_title = self._sigmund.condense_model.predict(
-            title_prompt).strip('"\'')
-        if len(self._conversation_title) > 100:
-            self._conversation_title = self._conversation_title[:100] + '…'
+        return '\n\n'.join(chunk for chunk in system_prompt if chunk.strip())    
 
     def load(self):
         try:
@@ -223,11 +209,39 @@ class Messages:
             self.save()
     
     def save(self):
-        self._update_title()
         conversation = {
             'condensed_text': self._condensed_text,
             'message_history': self._message_history,
             'condensed_message_history': self._condensed_message_history,
             'title': self._conversation_title,
         }
-        self._sigmund.database.update_active_conversation(conversation)
+        self._conversation_id = self._sigmund.database.update_active_conversation(conversation)
+        # We update the title in a background process so that we don't block
+        # the conversation
+        mp.Process(target=self._update_title).start()
+
+    def _update_title(self):
+        """The conversation title is updated when there are at least two 
+        messages, excluding the system prompt and AI welcome message. Based on
+        the last messages, a summary title is then created.
+    
+        This method is run in a separate process to avoid blocking the main thread.
+        """
+        if len(self) <= 2 or \
+                self._conversation_title != config.default_conversation_title:
+            return
+        logger.info('updating conversation title')
+        title_prompt = [dict(role='system', content=prompt.TITLE_PROMPT)]
+        title_prompt += self.prompt()[2:]
+        suggested_title = self._sigmund.condense_model.predict(title_prompt)
+        # The prediction may be a tool call, so we need to check if it is a str.
+        # This should not ordinarily happen, but sometimes models get confused.
+        if not isinstance(suggested_title, str):
+            logger.error(f'suggested conversation title is not str, but: {suggested_title}')
+            return
+        self._conversation_title = suggested_title.strip('"\'')
+        if len(self._conversation_title) > 100:
+            self._conversation_title = self._conversation_title[:100] + '…'
+        self._sigmund.database.set_conversation_title(
+            self._conversation_id, self._conversation_title)
+        logger.info('completed updating conversation title')
