@@ -12,23 +12,37 @@ logger = logging.getLogger('sigmund')
 api_blueprint = Blueprint('api', __name__)
 
 
+@api_blueprint.route('/health', methods=['GET'])
+@login_required
+def health():
+    return jsonify(success=True)
+
+
 @api_blueprint.route('/chat/start', methods=['POST'])
 @login_required
 def api_chat_start():
 
-    # For multipart/form-data, use request.form for text data
+    # Store request data in the session
     message = request.form.get('message', '')
     workspace_content = request.form.get('workspace_content', '')
     workspace_language = request.form.get('workspace_language', '')
     message_id = request.form.get('message_id', '')
-
-    # Example: store the user inputs in session
+    transient_settings = request.form.get('transient_settings', '{}')
+    transient_settings = json.loads(transient_settings)
+    transient_system_prompt = request.form.get('transient_system_prompt', '')
+    foundation_document_topics = request.form.get('foundation_document_topics',
+                                                  None)
+    if foundation_document_topics is not None:
+        foundation_document_topics = json.loads(foundation_document_topics)
     session['user_message']       = message
     session['workspace_content']  = workspace_content
     session['workspace_language'] = workspace_language
     session['message_id']         = message_id
+    session['transient_settings'] = transient_settings
+    session['transient_system_prompt'] = transient_system_prompt
+    session['foundation_document_topics'] = foundation_document_topics
 
-    sigmund = get_sigmund()
+    sigmund = get_sigmund(transient_settings=transient_settings)
     redis_client.delete(f'stream_cancel_{sigmund.user_id}')
 
     # Grab attached files from the request
@@ -73,9 +87,14 @@ def api_chat_stream():
     workspace_content = session.get('workspace_content', '')
     workspace_language = session.get('workspace_language', '')
     message_id = session.get('message_id', '')
+    transient_settings = session.get('transient_settings')
+    transient_system_prompt = session.get('transient_system_prompt')
+    foundation_document_topics = session.get('foundation_document_topics')
 
     # Retrieve attachments from Redis
-    sigmund = get_sigmund()
+    sigmund = get_sigmund(transient_settings=transient_settings,
+                          transient_system_prompt=transient_system_prompt,
+                          foundation_document_topics=foundation_document_topics)
     attachments_json = redis_client.get(f'attachments_{sigmund.user_id}')
     attachments = []
     if attachments_json:
